#define WIN32_LEAN_AND_MEAN
#define WINVER 0x0500
#include <windows.h>

static HWND hWndMain, hWndTool1, hWndTool2;

static LRESULT CALLBACK ToolWindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_CREATE:
    {
	HWND parent = (HWND)GetWindowLong(hWnd, GWL_HWNDPARENT);
	return 0;
    }

    case WM_LBUTTONUP:
    {
	HWND init_parent, init_owner, gwl1_parent, gp1_parent, ret_parent,
		 new_parent, new_owner, gwl2_parent, gp2_parent, root, rootowner;

	// The parent of tool window 2 should always be the desktop and the owner 
	// should be 0 initially, and the the first tool window
	init_parent = (HWND)GetAncestor(hWnd, GA_PARENT);
	init_owner = GetWindow(hWnd, GW_OWNER);

	// This will return the owner
	gwl1_parent = (HWND)GetWindowLong(hWnd, GWL_HWNDPARENT);
	// This will return 0
	gp1_parent = GetParent (hWnd);

	// This should actually change the owner of the second tool window to the first
	if (hWnd == hWndTool2)
	    ret_parent = (HWND)SetWindowLong(hWnd, GWL_HWNDPARENT, (LONG)hWndTool1);

	// The parent should still be the desktop
	new_parent = (HWND)GetAncestor(hWnd, GA_PARENT);
	// The owner should now by ToolWindow2 (NB not the main window)
	new_owner = GetWindow(hWnd, GW_OWNER);
	// This should give the owner
	gwl2_parent = (HWND)GetWindowLong(hWnd, GWL_HWNDPARENT);
	// This will return 0
	gp2_parent = GetParent (hWnd);
    // I expect this to return itself
	root = GetAncestor (hWnd, GA_ROOT);
	// I expect this to return itself since GetParent returns 0
	rootowner = GetAncestor (hWnd, GA_ROOTOWNER);

	break;
    }

    default: break;
    }

    return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

static BOOL CreateToolWindows(HWND parent)
{
    int x, y, width, height;
    DWORD style = WS_CAPTION | WS_SYSMENU;

	// Create the first window as a child of the main window
    x = 100;
    y = 300;
    width = height = 200;
    if (!(hWndTool1 = CreateWindowEx(0, "ToolWindowClass", "Tool window 1",
	style | WS_CHILD,
	x, y,
	width,
	height,
	parent,
	0,
	0,
	NULL)))
	return FALSE;

    ShowWindow(hWndTool1, SW_SHOWNORMAL);

	// Create the second window as a top level window
    x = 300;
    y = 100;
    width = height = 200;
    if(!(hWndTool2 = CreateWindowEx(WS_EX_APPWINDOW, "ToolWindowClass", "Tool window 2",
	style,
	x,
	y,
	width,
	height,
	0 /* Desktop */,
	0,
	0,
	NULL)))
	return FALSE;

    ShowWindow(hWndTool2, SW_SHOWNORMAL);

    return TRUE;
}

static LRESULT CALLBACK MainWindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_CREATE:
    {
	HWND parent = (HWND)GetWindowLong(hWnd, GWL_HWNDPARENT);
	HWND owner = GetWindow(hWnd, GW_OWNER);
	if (!CreateToolWindows(/* 0 */ hWnd))
	    return -1;
	return 0;
    }

    case WM_CLOSE:
	DestroyWindow(hWnd);
	return 1;

    case WM_DESTROY:
	PostQuitMessage(0);
	return 1;

    default: break;
    }

    return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

static BOOL RegisterWindowClasses(void)
{
    WNDCLASS cls;

    cls.style = 0;
    cls.lpfnWndProc = MainWindowProc;
    cls.cbClsExtra = 0;
    cls.cbWndExtra = 0;
    cls.hInstance = GetModuleHandle(0);
    cls.hIcon = 0;
    cls.hCursor = LoadCursor(0, IDC_ARROW);
    cls.hbrBackground = GetStockObject(WHITE_BRUSH);
    cls.lpszMenuName = NULL;
    cls.lpszClassName = "MainWindowClass";

    if(!RegisterClass(&cls))
	return FALSE;

    cls.style = 0;
    cls.lpfnWndProc = ToolWindowProc;
    cls.cbClsExtra = 0;
    cls.cbWndExtra = 0;
    cls.hInstance = GetModuleHandle(0);
    cls.hIcon = 0;
    cls.hCursor = LoadCursor(0, IDC_ARROW);
    cls.hbrBackground = GetStockObject(WHITE_BRUSH);
    cls.lpszMenuName = NULL;
    cls.lpszClassName = "ToolWindowClass";

    if(!RegisterClass(&cls))
	return FALSE;

    return TRUE;
}

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;

    if(!RegisterWindowClasses())
	return 0;

    if (!(hWndMain = CreateWindowEx(/*WS_EX_TOOLWINDOW*/ 0, "MainWindowClass", "Main window",
	WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_VISIBLE | WS_POPUP,
	100, 100,
	200,
	200,
	0, //GetDesktopWindow()
	0,
	0,
	NULL)))
	return -1;

    while (GetMessage(&msg, 0, 0, 0))
    {
	TranslateMessage(&msg);
	DispatchMessage(&msg);
    }

    return 0;
}
